-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
--  _______ _______ _______ _______    _______ _______ _______________________ ________________
-- (  ____ (  ___  |       |  ____ \  (  ____ |  ____ |  ___  )__    _(  ____ (  ____ \__   __/
-- | (    \/ (   ) | () () | (    \/  | (    )| (    )| (   ) |  )  ( | (    \/ (    \/  ) (   
-- | |     | (___) | || || | (__      | (____)| (____)| |   | |  |  | | (__   | |        | |   
-- | | ____|  ___  | |(_)| |  __)     |  _____)     __) |   | |  |  | |  __)  | |        | |   
-- | | \_  ) (   ) | |   | | (        | (     | (\ (  | |   | |  |  | | (     | |        | |   
-- | (___) | )   ( | )   ( | (____/\  | )     | ) \ \_| (___) |\_)  ) | (____/\ (____/\  | |   
-- (_______)/     \|/     \(_______/  |/      |/   \__(_______|____/  (_______(_______/  )_(   
--                    _______ _______ ______  ______ _________________
--                   (  ____ |  ____ (  __  \(  __  \\__   __|__   __/
--                   | (    )| (    \/ (  \  ) (  \  )  ) (     ) (   
--                   | (____)| (__   | |   ) | |   ) |  | |     | |   
--                   |     __)  __)  | |   | | |   | |  | |     | |   
--                   | (\ (  | (     | |   ) | |   ) |  | |     | |   
--                   | ) \ \_| (____/\ (__/  ) (__/  )__) (___  | |   
--                   |/   \__(_______(______/(______/\_______/  )_(                                                                                                                      
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
-- LOVE - Debug Info
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
debug = true
if debug then
  print("Debug Info:")
  print("OS: "..love.system.getOS( ))
  print("Power Supply: "..love.system.getPowerInfo( ))
  print("CPU Count: "..love.system.getProcessorCount( ))
end
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
-- LOVE - Game Settings and info
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
--Game Settings
game = {
  groundlevel = 60,
}

--Possible Resolutions
resolutions = {current = 1, use = false, timermax = 1.0, timercurrent = 0.0, scale = 1.0}
resolutions[1] = {width = 640, height = 360, fullscreen = false}
resolutions[2] = {width = 960, height = 540, fullscreen = false}
resolutions[3] = {width = 1280, height = 720, fullscreen = false}
resolutions[4] = {width = 1600, height = 900, fullscreen = false}
resolutions[5] = {width = 1920, height = 1080, fullscreen = false}
resolutions[6] = {width = 1920, height = 1080, fullscreen = true}

--Shader Settings
shader = {current = 0, use = false, timermax = 1.0, timercurrent = 0.0}

--Background
map = {groundlevel = 60, background = nil}
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
-- LOVE - Load; Load sprites, music, and basic settings
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function love.load(arg)
--Fonts
font_pixelsmall = love.graphics.newFont("assets/fonts/slkscr.ttf", 14)
font_pixelsmall:setFilter('nearest', 'nearest', 0)
font_pixellarge = love.graphics.newFont("assets/fonts/slkscre.ttf", 26)
font_pixellarge:setFilter('nearest', 'nearest', 0)
love.graphics.setFont(font_pixelsmall)

--Sprites
map.background = love.graphics.newImage('assets/backgrounds/test-room.png')
map.background:setFilter('nearest', 'nearest', 0)

--Shader
shader[1] = love.graphics.newShader("shaders/blackwhite.glsl")
shader[2] = love.graphics.newShader("shaders/scanlines.glsl")

--Canvas
canvas = love.graphics.newCanvas()
end
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
-- LOVE - Functions; Custom functions
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function playSound(sd)
  love.audio.newSource(sd, "static"):play()
end

function CheckCollision(x1,y1,w1,h1, x2,y2,w2,h2)
  return x1 < x2+w2 and
  x2 < x1+w1 and
  y1 < y2+h2 and
  y2 < y1+h1
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
-- LOVE - Update; Update positions, check for collisions, timers, and generating movement
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function love.update(dt)
  --Select Resolution
  resolutions.timercurrent = resolutions.timercurrent - dt
  if love.keyboard.isDown('1') then
    if resolutions.timercurrent < 0 then
      if resolutions.use and resolutions.current < 6 then
        resolutions.current = resolutions.current + 1
      elseif resolutions.use and resolutions.current > 4 then
        resolutions.current = 1
        resolutions.use = false
      else
        resolutions.use = true
      end
      if debug then
        print("Screen Mode:"..tostring(resolutions.current))
      end
      love.window.setMode(resolutions[resolutions.current].width, resolutions[resolutions.current].height, {fullscreen = resolutions[resolutions.current].fullscreen, vsync=false})
      resolutions.scale = love.graphics.getWidth() / 640
      resolutions.timercurrent = resolutions.timermax
    end
  end

  --Select Shader
  shader.timercurrent = shader.timercurrent - dt
  if love.keyboard.isDown('2') then
    if shader.timercurrent < 0 then
      if shader.use and shader.current < 2 then
        shader.current = shader.current + 1
      elseif shader.use and shader.current > 1 then
        shader.current = 0
        shader.use = false
      else
        shader.use = true
      end
      if debug then
        print("Shader Mode:"..tostring(shader.current))
      end
      shader.timercurrent = shader.timermax
    end
  end
end
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
-- LOVE - Draw; Draw and re-draw sprites, UI and background
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function love.draw(dt)
  love.graphics.push()
  
  --Apply Ratio
  love.graphics.scale(resolutions.scale, resolutions.scale)
  
  --Cleanup Canvas
  canvas:clear()
  love.graphics.setCanvas(canvas)
  
  --Background Image
  love.graphics.draw(map.background)
  
  --Testchar
  love.graphics.rectangle("fill", 360 - map.groundlevel, 240, 20, 60)
  
  --FPS Display
  love.graphics.print(tostring(love.timer.getFPS( )), 620, 00)
  
  love.graphics.pop()
  
  --Re-Draw Canvas
  if shader.use then
    love.graphics.setShader(shader[shader.current])
  end
  love.graphics.draw(canvas)
  love.graphics.setShader()
end